package negocio;

import java.awt.Color;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.CategoryLabelPositions;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PiePlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.chart.renderer.category.StandardBarPainter;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.data.general.DefaultPieDataset;

/**
 * Clase encargada de acceder al padrn y generar instancias de ResultadoConsulta a partir de peticiones.
 * 
 * @author Santiago de la Parte Flrez
 */
public class Padron {
	
	// Constantes
	final private String phpURL = "http://gutenberg.dcs.fi.uva.es/~santiago/appletphp/consultas.php";
	final private Color colorProcedencia = new Color(Integer.parseInt("3B0B17", 16));
	final private Color colorEstudios = new Color(Integer.parseInt("61210B", 16));
	final private Color colorSexoM = new Color(Integer.parseInt("F5F6CE", 16));
	final private Color colorSexoH = new Color(Integer.parseInt("0B173B", 16));
	final private Color colorEdadM = new Color(Integer.parseInt("610B21", 16));
	final private Color colorEdadH = new Color(Integer.parseInt("243B0B", 16));
	private int poblacion = -1;

	/**
	 * Creation
	 */
	protected Padron() {}
	
	/**
	 * 
	 * @author Santiago de la Parte Flrez
	 * 
	 * Enumerador del tipo de procendencia
	 *
	 */
	protected static enum TipoProcedencia {
		 NACIONAL, EXTRANJERA;  //; is optional
	}
	
	/**
	 * 
	 * Devuelve resultado de consulta sobre procedencia a partir de los parmetros.
	 * 
	 * @param distrito Nmero de distrito. -1 para indicar "ninguno".
	 * @param seccion Nmero de seccin. -1 para indicar "ninguno".
	 * @param zona Zona estadstica. null para indicar "ninguna".
	 * @param sexo Sexo.
	 * @param estudios Estudios.
	 * @param edad Edad.
	 * @param tipoP Tipo de procedencia.
	 * @return Instancia de ResultadoConsulta
	 */
	protected ResultadoConsulta consultaProcedencia(int distrito, int seccion, String zona, String sexo, String estudios, String edad, TipoProcedencia tipoP) {
		String title = "Procedencia ";
		if (tipoP == TipoProcedencia.NACIONAL)
			title += "(espaoles) ";
		else
			title += "(extranjeros) ";
		
		if (distrito!=-1) {
			title += "Distrito " + distrito;
			if (seccion!=-1) {
				title += ", Seccin " + seccion;
			}
		} else {
			if (zona!=null) {
				title += "Zona "+zona;
			} else {
				title += "Valladolid";
			}
		}
		
		// Creo el DataSet
		DefaultCategoryDataset dataset = new DefaultCategoryDataset();

		String tipo = "";
		if (tipoP == TipoProcedencia.NACIONAL)
			tipo = "nacional";
		else
			tipo = "extranjero";
		String miSexo = null; 
		String miNacional = null; 
		String miExtranjero = null;
		String miEdad = null;
		boolean miEspanha = false; 
		String miEstudios = null; 
		int miDistrito = distrito;
		int miSeccion = seccion;
		String miZona = zona;
		
		// Preparar estudios
		switch(estudios) {
		case "Sin estudios":
			miEstudios = "SINES";
			break;
		case "Estudios Primarios":
			miEstudios = "PRIMA";
			break;
		case "Enseanza Profesional de 2 Grado (Segundo Ciclo)":
			miEstudios = "PROF2";
			break;
		case "Enseanza Secundaria (Primer Ciclo)":
			miEstudios = "SECU1";
			break;
		case "Enseanza Profesional Superior":
			miEstudios = "PROFS";
			break;
		case "Enseanza Secundaria (Segundo Ciclo)":
			miEstudios = "SECU2";
			break;
		case "Estudios Universitarios o superiores":
			miEstudios = "UNIVE";
			break;
		case "Desconocido":
			miEstudios = "DESCO";
			break;
		}
		
		// Preparar edad
		switch(edad) {
		case "Todos":
			break;
		case "85+":
			miEdad = "+85";
			break;
		default:
			miEdad = edad;
			break;
		}

		// Preparar sexo
		switch(sexo) {
		case "Todos":
			break;
		case "Varn":
			miSexo = "varon";
			break;
		default:
			miSexo = "mujer";
			break;
		}
		
		String miResultado = enviarDatos(tipo, miSexo, miNacional, miExtranjero, miEdad, miEspanha, miEstudios, miDistrito, miSeccion, miZona);

		// Interpreto el resultado devuelto por enviarDatos
		if (miResultado.length()>2)
			miResultado = miResultado.substring(0, miResultado.length()-2);
		String [] r = miResultado.split("@");
		
		ResultadoConsulta miResultadoConsulta = new ResultadoConsulta();
		
		// Preparo el resultado para una consulta sobre procedencia nacional
		if (tipoP == TipoProcedencia.NACIONAL) {
			
			//Andaluca, Aragn, Asturias, Baleares, Canarias, Cantabria, Castilla La-Mancha, Castilla y Len, Catalua,  Cdad. Valenciana, Extremadura, Galicia, La Rioja, Cdad. de Madrid, Navarra, Pas Vasco, Murcia, Ceuta, Melilla, Extranjero
			int[] valores = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
			
			for (int j=0; j<r.length; j++) {
				String [] s = r[j].split("#");
				switch (s[0]) {
				case "Andaluca":
					valores[0] = Integer.parseInt(s[1]);
					break;
				case "Aragn":
					valores[1] = Integer.parseInt(s[1]);
					break;
				case "Principado de Asturias":
					valores[2] = Integer.parseInt(s[1]);
					break;
				case "Islas Baleares":
					valores[3] = Integer.parseInt(s[1]);
					break;
				case "Canarias":
					valores[4] = Integer.parseInt(s[1]);
					break;
				case "Cantabria":
					valores[5] = Integer.parseInt(s[1]);
					break;
				case "Castilla y Len":
					valores[6] = Integer.parseInt(s[1]);
					break;
				case "Castilla-La Mancha":
					valores[7] = Integer.parseInt(s[1]);
					break;
				case "Catalua":
					valores[8] = Integer.parseInt(s[1]);
					break;
				case "Extremadura":
					valores[9] = Integer.parseInt(s[1]);
					break;
				case "Galicia":
					valores[10] = Integer.parseInt(s[1]);
					break;
				case "Comunidad de Madrid":
					valores[11] = Integer.parseInt(s[1]);
					break;
				case "Regin de Murcia":
					valores[12] = Integer.parseInt(s[1]);
					break;
				case "Navarra":
					valores[13] = Integer.parseInt(s[1]);
					break;
				case "La Rioja":
					valores[14] = Integer.parseInt(s[1]);
					break;
				case "Pas Vasco":
					valores[15] = Integer.parseInt(s[1]);
					break;
				case "Comunidad Valenciana":
					valores[16] = Integer.parseInt(s[1]);
					break;
				case "Ceuta":
					valores[17] = Integer.parseInt(s[1]);
					break;
				case "Melilla":
					valores[18] = Integer.parseInt(s[1]);
					break;
				case "Extranjero":
					valores[19] = Integer.parseInt(s[1]);
					break;
				case "Desconocida":
					valores[20] = Integer.parseInt(s[1]);
					break;
				}
			}
			
			// Doy valores a los elementos del dataset
			dataset.addValue(valores[0], "Poblacin", "Andaluca");
			dataset.addValue(valores[1], "Poblacin", "Aragn");
			dataset.addValue(valores[2], "Poblacin", "Asturias");
			dataset.addValue(valores[3], "Poblacin", "Baleares");
			dataset.addValue(valores[4], "Poblacin", "Canarias");
			dataset.addValue(valores[5], "Poblacin", "Cantabria");
			dataset.addValue(valores[6], "Poblacin", "Castilla y Len");
			dataset.addValue(valores[7], "Poblacin", "Castilla La-Mancha");
			dataset.addValue(valores[8], "Poblacin", "Catalua");
			dataset.addValue(valores[9], "Poblacin", "Extremadura");
			dataset.addValue(valores[10], "Poblacin", "Galicia");
			dataset.addValue(valores[11], "Poblacin", "Cdad. de Madrid");
			dataset.addValue(valores[12], "Poblacin", "Murcia");
			dataset.addValue(valores[13], "Poblacin", "Navarra");
			dataset.addValue(valores[14], "Poblacin", "La Rioja");
			dataset.addValue(valores[15], "Poblacin", "Pas Vasco");
			dataset.addValue(valores[16], "Poblacin", "Cdad. Valenciana");
			dataset.addValue(valores[17], "Poblacin", "Ceuta");
			dataset.addValue(valores[18], "Poblacin", "Melilla");
			dataset.addValue(valores[19], "Poblacin", "Extranjero");
			dataset.addValue(valores[20], "Poblacin", "Desconocida");
			
			// Calculo la poblacion representada
			int poblacionCons = 0;
			for (int i=0; i<valores.length; i++) {
				poblacionCons += valores[i];
			}
			miResultadoConsulta.setPoblacionConsulta(poblacionCons);
		
		// Preparo el resultado para una consulta sobre procedencia nacional
		} else {
			
			//Europa, Africa, America del Norte, Amrica Central, Amrica del Sur, Asia, Oceana, Aptrida
			int[] valores = {0,0,0,0,0,0,0,0,0};
			
			for (int j=0; j<r.length; j++) {
				String [] s = r[j].split("#");
				switch (s[0]) {
				case "EURO":
					valores[0] = Integer.parseInt(s[1]);
					break;
				case "AFRI":
					valores[1] = Integer.parseInt(s[1]);
					break;
				case "AMNO":
					valores[2] = Integer.parseInt(s[1]);
					break;
				case "AMCE":
					valores[3] = Integer.parseInt(s[1]);
					break;
				case "AMSU":
					valores[4] = Integer.parseInt(s[1]);
					break;
				case "ASIA":
					valores[5] = Integer.parseInt(s[1]);
					break;
				case "OCEA":
					valores[6] = Integer.parseInt(s[1]);
					break;
				case "APAT":
					valores[7] = Integer.parseInt(s[1]);
					break;
				case "DESC":
					valores[8] = Integer.parseInt(s[1]);
					break;
				}
			}
			
			// Doy valores a los elementos del dataset
			dataset.addValue(valores[0], "Poblacin", "Europa");
			dataset.addValue(valores[1], "Poblacin", "frica");
			dataset.addValue(valores[2], "Poblacin", "Amrica del Norte");
			dataset.addValue(valores[3], "Poblacin", "Amrica Central");
			dataset.addValue(valores[4], "Poblacin", "Amrica del Sur");
			dataset.addValue(valores[5], "Poblacin", "Asia");
			dataset.addValue(valores[6], "Poblacin", "Oceana");
			dataset.addValue(valores[7], "Poblacin", "Aptrida");
			dataset.addValue(valores[7], "Poblacin", "Desconocida");
			
			// Calculo la poblacion representada
			int poblacionCons = 0;
			for (int i=0; i<valores.length; i++) {
				poblacionCons += valores[i];
			}
			miResultadoConsulta.setPoblacionConsulta(poblacionCons);
		}
		
		// Creo la grfica
		JFreeChart chart = ChartFactory.createBarChart(title, null, "Poblacin", dataset, PlotOrientation.VERTICAL, false, true, false);
		// Configuro la inclinacin del texto del eje inferior
		CategoryPlot plot = (CategoryPlot) chart.getPlot();
		CategoryAxis domainAxis = plot.getDomainAxis();
		domainAxis.setCategoryLabelPositions(CategoryLabelPositions.UP_45);
		// Configuro el color y estilo de las barras
		BarRenderer renderer = (BarRenderer) plot.getRenderer();
		renderer.setSeriesPaint(0, colorProcedencia);
		renderer.setBarPainter(new StandardBarPainter());
		// Creo el resultado, aado los valores y lo devuelvo
		miResultadoConsulta.setChart(chart);
		if (poblacion==-1)
			poblacion = consultaPoblacionTotal();
		miResultadoConsulta.setPoblacion(poblacion);
		return miResultadoConsulta;
	}
	
	/**
	 * 
	 * Devuelve resultado de consulta sobre sexo a partir de los parmetros.
	 * 
	 * @param distrito Nmero de distrito. -1 para indicar "ninguno".
	 * @param seccion Nmero de seccin. -1 para indicar "ninguno".
	 * @param zona Zona estadstica. null para indicar "ninguna".
	 * @param procedencia Procedencia.
	 * @param estudios Estudios.
	 * @param edad Edad.
	 * @return Instancia de ResultadoConsulta
	 */
	protected ResultadoConsulta consultaSexo(int distrito, int seccion, String zona, String procedencia, String estudios, String edad) {
		String title = "Sexo ";
		if (distrito!=-1) {
			title += "Distrito " + distrito;
			if (seccion!=-1) {
				title += ", Seccin " + seccion;
			}
		} else {
			if (zona!=null) {
				title += "Zona "+zona;
			} else {
				title += "Valladolid";
			}
		}
		
		// Creo el DataSet
		DefaultPieDataset dataset = new DefaultPieDataset();
		
		String tipo = "sexo"; 
		String miSexo = null; 
		String miNacional = null; 
		String miExtranjero = null;
		String miEdad = null;
		boolean miEspanha = false; 
		String miEstudios = null; 
		int miDistrito = distrito;
		int miSeccion = seccion;
		String miZona = zona;
		
		switch (procedencia) {
		case "Espaa":
			miEspanha = true;
			break;
		case "Andaluca":
			miNacional = "AND";
			break;
		case "Aragn":
			miNacional = "ARA";
			break;
		case "Principado de Asturias":
			miNacional = "AST";
			break;
		case "Baleares":
			miNacional = "BAL";
			break;
		case "Canarias":
			miNacional = "CAN";
			break;
		case "Cantabria":
			miNacional = "CBR";
			break;
		case "Castilla-La Mancha":
			miNacional = "CLM";
			break;
		case "Castilla y Len":
			miNacional = "CYL";
			break;
		case "Catalua":
			miNacional = "CAT";
			break;
		case "Comunidad Valenciana":
			miNacional = "VAL";
			break;
		case "Extremadura":
			miNacional = "EXT";
			break;
		case "Galicia":
			miNacional = "GAL";
			break;
		case "La Rioja":
			miNacional = "RIO";
			break;
		case "Comunidad de Madrid":
			miNacional = "MAD";
			break;
		case "Navarra":
			miNacional = "NAV";
			break;
		case "Pas Vasco":
			miNacional = "PVA";
			break;
		case "Regin de Murcia":
			miNacional = "MUR";
			break;
		case "Ceuta":
			miNacional = "CEU";
			break;
		case "Melilla":
			miNacional = "MEL";
			break;
		case "Europa":
			miExtranjero = "EURO";
			break;
		case "frica":
			miExtranjero = "AFRI";
			break;
		case "Amrica del Norte":
			miExtranjero = "AMNO";
			break;
		case "Amrica Central":
			miExtranjero = "AMCE";
			break;
		case "Amrica del Sur":
			miExtranjero = "AMSU";
			break;
		case "Asia":
			miExtranjero = "ASIA";
			break;
		case "Oceana":
			miExtranjero = "OCEA";
			break;
		case "Aptrida":
			miExtranjero = "APAT";
			break;
		}
		
		switch(estudios) {
		case "Sin estudios":
			miEstudios = "SINES";
			break;
		case "Estudios Primarios":
			miEstudios = "PRIMA";
			break;
		case "Enseanza Profesional de 2 Grado (Segundo Ciclo)":
			miEstudios = "PROF2";
			break;
		case "Enseanza Secundaria (Primer Ciclo)":
			miEstudios = "SECU1";
			break;
		case "Enseanza Profesional Superior":
			miEstudios = "PROFS";
			break;
		case "Enseanza Secundaria (Segundo Ciclo)":
			miEstudios = "SECU2";
			break;
		case "Estudios Universitarios o superiores":
			miEstudios = "UNIVE";
			break;
		case "Desconocido":
			miEstudios = "DESCO";
			break;
		}

		switch(edad) {
		case "Todos":
			break;
		case "85+":
			miEdad = "+85";
			break;
		default:
			miEdad = edad;
			break;
		}
		
		int[] valores = {0,0}; //Varones, Mujeres
		
		String miResultado = enviarDatos(tipo, miSexo, miNacional, miExtranjero, miEdad, miEspanha, miEstudios, miDistrito, miSeccion, miZona);

		if (miResultado.length()>2)
			miResultado = miResultado.substring(0, miResultado.length()-2);
		String [] r = miResultado.split("@");
		for (int j=0; j<r.length; j++) {
			String [] s = r[j].split("#");
			if (s[0].equals("varon")) {
				valores[0] = Integer.parseInt(s[1]);
			}
			if (s[0].equals("mujer")) {
				valores[1] = Integer.parseInt(s[1]);
			}
		}
		
		dataset.setValue("Varones", valores[0]);
		dataset.setValue("Mujeres", valores[1]);
		
		// Creo la grfica
		JFreeChart chart = ChartFactory.createPieChart3D(title, dataset, true, true, false);
		// Configuro el color y estilo de los sectores
		PiePlot plot = (PiePlot) chart.getPlot();
		plot.setLabelGenerator(null);
		plot.setCircular(true);
		plot.setSectionPaint("Varones", colorSexoH);
		plot.setSectionPaint("Mujeres", colorSexoM);
		// Creo el resultado, aado los valores y lo devuelvo
		ResultadoConsulta miResultadoConsulta = new ResultadoConsulta();
		miResultadoConsulta.setChart(chart);
		int poblacionCons = 0;
		for (int i=0; i<valores.length; i++) {
			poblacionCons += valores[i];
		}
		miResultadoConsulta.setPoblacionConsulta(poblacionCons);
		if (poblacion==-1)
			poblacion = consultaPoblacionTotal();
		miResultadoConsulta.setPoblacion(poblacion);
		return miResultadoConsulta;
	}

	/**
	 * 
	 * Devuelve resultado de consulta sobre edad a partir de los parmetros.
	 * 
	 * @param distrito Nmero de distrito. -1 para indicar "ninguno".
	 * @param seccion Nmero de seccin. -1 para indicar "ninguno".
	 * @param zona Zona estadstica. null para indicar "ninguna".
	 * @param sexo Sexo.
	 * @param procedencia Procedencia.
	 * @param estudios Estudios.
	 * @return Instancia de ResultadoConsulta
	 */
	protected ResultadoConsulta consultaEdad(int distrito, int seccion, String zona, String sexo, String procedencia, String estudios) {
		String title = "Edad ";
		if (distrito!=-1) {
			title += "Distrito " + distrito;
			if (seccion!=-1) {
				title += ", Seccin " + seccion;
			}
		} else {
			if (zona!=null) {
				title += "Zona "+zona;
			} else {
				title += "Valladolid";
			}
		}
		// Creo el DataSet
		DefaultCategoryDataset dataset = new DefaultCategoryDataset();
		//"0-4","5-9","10-14","15-19","20-24","25-29","30-34","35-39","40-44","45-49",
		//"50-54","55-59","60-64","65-69","70-74","75-79","80-84","85+"

		String tipo = "edad"; 
		String miSexo = null; 
		String miNacional = null; 
		String miExtranjero = null;
		String miEdad = null;
		boolean miEspanha = false; 
		String miEstudios = null; 
		int miDistrito = distrito;
		int miSeccion = seccion;
		String miZona = zona;
		
		switch (procedencia) {
		case "Espaa":
			miEspanha = true;
			break;
		case "Andaluca":
			miNacional = "AND";
			break;
		case "Aragn":
			miNacional = "ARA";
			break;
		case "Principado de Asturias":
			miNacional = "AST";
			break;
		case "Baleares":
			miNacional = "BAL";
			break;
		case "Canarias":
			miNacional = "CAN";
			break;
		case "Cantabria":
			miNacional = "CBR";
			break;
		case "Castilla-La Mancha":
			miNacional = "CLM";
			break;
		case "Castilla y Len":
			miNacional = "CYL";
			break;
		case "Catalua":
			miNacional = "CAT";
			break;
		case "Comunidad Valenciana":
			miNacional = "VAL";
			break;
		case "Extremadura":
			miNacional = "EXT";
			break;
		case "Galicia":
			miNacional = "GAL";
			break;
		case "La Rioja":
			miNacional = "RIO";
			break;
		case "Comunidad de Madrid":
			miNacional = "MAD";
			break;
		case "Navarra":
			miNacional = "NAV";
			break;
		case "Pas Vasco":
			miNacional = "PVA";
			break;
		case "Regin de Murcia":
			miNacional = "MUR";
			break;
		case "Ceuta":
			miNacional = "CEU";
			break;
		case "Melilla":
			miNacional = "MEL";
			break;
		case "Europa":
			miExtranjero = "EURO";
			break;
		case "frica":
			miExtranjero = "AFRI";
			break;
		case "Amrica del Norte":
			miExtranjero = "AMNO";
			break;
		case "Amrica Central":
			miExtranjero = "AMCE";
			break;
		case "Amrica del Sur":
			miExtranjero = "AMSU";
			break;
		case "Asia":
			miExtranjero = "ASIA";
			break;
		case "Oceana":
			miExtranjero = "OCEA";
			break;
		case "Aptrida":
			miExtranjero = "APAT";
			break;
		}
		
		switch(estudios) {
		case "Sin estudios":
			miEstudios = "SINES";
			break;
		case "Estudios Primarios":
			miEstudios = "PRIMA";
			break;
		case "Enseanza Profesional de 2 Grado (Segundo Ciclo)":
			miEstudios = "PROF2";
			break;
		case "Enseanza Secundaria (Primer Ciclo)":
			miEstudios = "SECU1";
			break;
		case "Enseanza Profesional Superior":
			miEstudios = "PROFS";
			break;
		case "Enseanza Secundaria (Segundo Ciclo)":
			miEstudios = "SECU2";
			break;
		case "Estudios Universitarios o superiores":
			miEstudios = "UNIVE";
			break;
		case "Desconocido":
			miEstudios = "DESCO";
			break;
		}

		switch(sexo) {
		case "Todos":
			break;
		case "Varn":
			miSexo = "varon";
			break;
		default:
			miSexo = "mujer";
			break;
		}
		
		int[] valoresH = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
		int[] valoresM = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
		//0, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60, 65, 70, 75, 80, 85
		
		String miResultado = enviarDatos(tipo, miSexo, miNacional, miExtranjero, miEdad, miEspanha, miEstudios, miDistrito, miSeccion, miZona);
		if (miResultado.length()>2)
			miResultado = miResultado.substring(0, miResultado.length()-2);
		String [] r = miResultado.split("@");
		int edad = -1;
		int valor = 0;
		String sexoConsulta = "";
		
		for (int j=0; j<r.length; j++) {
			String [] s = r[j].split("#");
			if (s.length < 3)
				continue;
			edad = Integer.parseInt(s[0]);
			sexoConsulta = s[1];
			valor = Integer.parseInt(s[2]);
			switch(edad) {
			case 0:
			case 1:
			case 2:
			case 3:
			case 4:
				if (sexoConsulta.equals("varon"))
					valoresH[0] = valoresH[0] + valor;
				else
					valoresM[0] = valoresM[0] + valor;
				break;
			case 5:
			case 6:
			case 7:
			case 8:
			case 9:
				if (sexoConsulta.equals("varon"))
					valoresH[1] = valoresH[1] + valor;
				else
					valoresM[1] = valoresM[1] + valor;
				break;
			case 10:
			case 11:
			case 12:
			case 13:
			case 14:
				if (sexoConsulta.equals("varon"))
					valoresH[2] = valoresH[2] + valor;
				else
					valoresM[2] = valoresM[2] + valor;
				break;
			case 15:
			case 16:
			case 17:
			case 18:
			case 19:
				if (sexoConsulta.equals("varon"))
					valoresH[3] = valoresH[3] + valor;
				else
					valoresM[3] = valoresM[3] + valor;
				break;
			case 20:
			case 21:
			case 22:
			case 23:
			case 24:
				if (sexoConsulta.equals("varon"))
					valoresH[4] = valoresH[4] + valor;
				else
					valoresM[4] = valoresM[4] + valor;
				break;
			case 25:
			case 26:
			case 27:
			case 28:
			case 29:
				if (sexoConsulta.equals("varon"))
					valoresH[5] = valoresH[5] + valor;
				else
					valoresM[5] = valoresM[5] + valor;
				break;
			case 30:
			case 31:
			case 32:
			case 33:
			case 34:
				if (sexoConsulta.equals("varon"))
					valoresH[6] = valoresH[6] + valor;
				else
					valoresM[6] = valoresM[6] + valor;
				break;
			case 35:
			case 36:
			case 37:
			case 38:
			case 39:
				if (sexoConsulta.equals("varon"))
					valoresH[7] = valoresH[7] + valor;
				else
					valoresM[7] = valoresM[7] + valor;
				break;
			case 40:
			case 41:
			case 42:
			case 43:
			case 44:
				if (sexoConsulta.equals("varon"))
					valoresH[8] = valoresH[8] + valor;
				else
					valoresM[8] = valoresM[8] + valor;
				break;
			case 45:
			case 46:
			case 47:
			case 48:
			case 49:
				if (sexoConsulta.equals("varon"))
					valoresH[9] = valoresH[9] + valor;
				else
					valoresM[9] = valoresM[9] + valor;
				break;
			case 50:
			case 51:
			case 52:
			case 53:
			case 54:
				if (sexoConsulta.equals("varon"))
					valoresH[10] = valoresH[10] + valor;
				else
					valoresM[10] = valoresM[10] + valor;
				break;
			case 55:
			case 56:
			case 57:
			case 58:
			case 59:
				if (sexoConsulta.equals("varon"))
					valoresH[11] = valoresH[11] + valor;
				else
					valoresM[11] = valoresM[11] + valor;
				break;
			case 60:
			case 61:
			case 62:
			case 63:
			case 64:
				if (sexoConsulta.equals("varon"))
					valoresH[12] = valoresH[12] + valor;
				else
					valoresM[12] = valoresM[12] + valor;
				break;
			case 65:
			case 66:
			case 67:
			case 68:
			case 69:
				if (sexoConsulta.equals("varon"))
					valoresH[13] = valoresH[13] + valor;
				else
					valoresM[13] = valoresM[13] + valor;
				break;
			case 70:
			case 71:
			case 72:
			case 73:
			case 74:
				if (sexoConsulta.equals("varon"))
					valoresH[14] = valoresH[14] + valor;
				else
					valoresM[14] = valoresM[14] + valor;
				break;
			case 75:
			case 76:
			case 77:
			case 78:
			case 79:
				if (sexoConsulta.equals("varon"))
					valoresH[15] = valoresH[15] + valor;
				else
					valoresM[15] = valoresM[15] + valor;
				break;
			case 80:
			case 81:
			case 82:
			case 83:
			case 84:
				if (sexoConsulta.equals("varon"))
					valoresH[16] = valoresH[16] + valor;
				else
					valoresM[16] = valoresM[16] + valor;
				break;
			default:
				if (sexoConsulta.equals("varon"))
					valoresH[17] = valoresH[17] + valor;
				else
					valoresM[17] = valoresM[17] + valor;
				break;
			}
		}
		
		dataset.addValue((valoresM[17]*-1), "Mujeres", "85+");
		dataset.addValue((valoresM[16]*-1), "Mujeres", "80-84");
		dataset.addValue((valoresM[15]*-1), "Mujeres", "75-79");
		dataset.addValue((valoresM[14]*-1), "Mujeres", "70-74");
		dataset.addValue((valoresM[13]*-1), "Mujeres", "65-69");
		dataset.addValue((valoresM[12]*-1), "Mujeres", "60-64");
		dataset.addValue((valoresM[11]*-1), "Mujeres", "55-59");
		dataset.addValue((valoresM[10]*-1), "Mujeres", "50-54");
		dataset.addValue((valoresM[9]*-1), "Mujeres", "45-49");
		dataset.addValue((valoresM[8]*-1), "Mujeres", "40-44");
		dataset.addValue((valoresM[7]*-1), "Mujeres", "35-39");
		dataset.addValue((valoresM[6]*-1), "Mujeres", "30-34");
		dataset.addValue((valoresM[5]*-1), "Mujeres", "25-29");
		dataset.addValue((valoresM[4]*-1), "Mujeres", "20-24");
		dataset.addValue((valoresM[3]*-1), "Mujeres", "15-19");
		dataset.addValue((valoresM[2]*-1), "Mujeres", "10-14");
		dataset.addValue((valoresM[1]*-1), "Mujeres", "5-9");
		dataset.addValue((valoresM[0]*-1), "Mujeres", "0-4");
		dataset.addValue(valoresH[17], "Varones", "85+");
		dataset.addValue(valoresH[16], "Varones", "80-84");
		dataset.addValue(valoresH[15], "Varones", "75-79");
		dataset.addValue(valoresH[14], "Varones", "70-74");
		dataset.addValue(valoresH[13], "Varones", "65-69");
		dataset.addValue(valoresH[12], "Varones", "60-64");
		dataset.addValue(valoresH[11], "Varones", "55-59");
		dataset.addValue(valoresH[10], "Varones", "50-54");
		dataset.addValue(valoresH[9], "Varones", "45-49");
		dataset.addValue(valoresH[8], "Varones", "40-44");
		dataset.addValue(valoresH[7], "Varones", "35-39");
		dataset.addValue(valoresH[6], "Varones", "30-34");
		dataset.addValue(valoresH[5], "Varones", "25-29");
		dataset.addValue(valoresH[4], "Varones", "20-24");
		dataset.addValue(valoresH[3], "Varones", "15-19");
		dataset.addValue(valoresH[2], "Varones", "10-14");
		dataset.addValue(valoresH[1], "Varones", "5-9");
		dataset.addValue(valoresH[0], "Varones", "0-4");
		
		
		// Creo la grfica
		JFreeChart chart = ChartFactory.createStackedBarChart(title,"Edad","Individuos",dataset,PlotOrientation.HORIZONTAL,true,true,false);
		// Configuro los colores
		CategoryPlot plot = (CategoryPlot) chart.getPlot();
		CategoryAxis domainAxis = plot.getDomainAxis();
		domainAxis.setCategoryMargin(0.05);
		BarRenderer renderer = (BarRenderer) plot.getRenderer();
		renderer.setSeriesPaint(0, colorEdadH);
		renderer.setSeriesPaint(1, colorEdadM);
		renderer.setBarPainter(new StandardBarPainter());
		// Creo el resultado, aado los valores y lo devuelvo
		ResultadoConsulta miResultadoConsulta = new ResultadoConsulta();
		miResultadoConsulta.setChart(chart);
		int poblacionCons = 0;
		for (int i=0; i<valoresH.length; i++) {
			poblacionCons += valoresH[i];
		}
		for (int i=0; i<valoresM.length; i++) {
			poblacionCons += valoresM[i];
		}
		miResultadoConsulta.setPoblacionConsulta(poblacionCons);
		if (poblacion==-1)
			poblacion = consultaPoblacionTotal();
		miResultadoConsulta.setPoblacion(poblacion);
		return miResultadoConsulta;
	}
	
	/**
	 * 
	 * Devuelve resultado de consulta sobre estudios a partir de los parmetros.
	 * 
	 * @param distrito Nmero de distrito. -1 para indicar "ninguno".
	 * @param seccion Nmero de seccin. -1 para indicar "ninguno".
	 * @param zona Zona estadstica. null para indicar "ninguna".
	 * @param sexo Sexo.
	 * @param procedencia Procedencia.
	 * @param edad Edad.
	 * @return Instancia de ResultadoConsulta
	 */
	protected ResultadoConsulta consultaEstudios(int distrito, int seccion, String zona, String sexo, String procedencia, String edad) {
		String title = "Estudios ";
		if (distrito!=-1) {
			title += "Distrito " + distrito;
			if (seccion!=-1) {
				title += ", Seccin " + seccion;
			}
		} else {
			if (zona!=null) {
				title += "Zona "+zona;
			} else {
				title += "Valladolid";
			}
		}
		// Creo el DataSet
		DefaultCategoryDataset dataset = new DefaultCategoryDataset();
		//"0-4","5-9","10-14","15-19","20-24","25-29","30-34","35-39","40-44","45-49",
		//"50-54","55-59","60-64","65-69","70-74","75-79","80-84","85+"

		String tipo = "estudios"; 
		String miSexo = null; 
		String miNacional = null; 
		String miExtranjero = null;
		String miEdad = null;
		boolean miEspanha = false; 
		String miEstudios = null; 
		int miDistrito = distrito;
		int miSeccion = seccion;
		String miZona = zona;
		
		switch (procedencia) {
		case "Espaa":
			miEspanha = true;
			break;
		case "Andaluca":
			miNacional = "AND";
			break;
		case "Aragn":
			miNacional = "ARA";
			break;
		case "Principado de Asturias":
			miNacional = "AST";
			break;
		case "Baleares":
			miNacional = "BAL";
			break;
		case "Canarias":
			miNacional = "CAN";
			break;
		case "Cantabria":
			miNacional = "CBR";
			break;
		case "Castilla-La Mancha":
			miNacional = "CLM";
			break;
		case "Castilla y Len":
			miNacional = "CYL";
			break;
		case "Catalua":
			miNacional = "CAT";
			break;
		case "Comunidad Valenciana":
			miNacional = "VAL";
			break;
		case "Extremadura":
			miNacional = "EXT";
			break;
		case "Galicia":
			miNacional = "GAL";
			break;
		case "La Rioja":
			miNacional = "RIO";
			break;
		case "Comunidad de Madrid":
			miNacional = "MAD";
			break;
		case "Navarra":
			miNacional = "NAV";
			break;
		case "Pas Vasco":
			miNacional = "PVA";
			break;
		case "Regin de Murcia":
			miNacional = "MUR";
			break;
		case "Ceuta":
			miNacional = "CEU";
			break;
		case "Melilla":
			miNacional = "MEL";
			break;
		case "Europa":
			miExtranjero = "EURO";
			break;
		case "frica":
			miExtranjero = "AFRI";
			break;
		case "Amrica del Norte":
			miExtranjero = "AMNO";
			break;
		case "Amrica Central":
			miExtranjero = "AMCE";
			break;
		case "Amrica del Sur":
			miExtranjero = "AMSU";
			break;
		case "Asia":
			miExtranjero = "ASIA";
			break;
		case "Oceana":
			miExtranjero = "OCEA";
			break;
		case "Aptrida":
			miExtranjero = "APAT";
			break;
		}
		
		switch(edad) {
		case "Todos":
			break;
		case "85+":
			miEdad = "+85";
			break;
		default:
			miEdad = edad;
			break;
		}

		switch(sexo) {
		case "Todos":
			break;
		case "Varn":
			miSexo = "varon";
			break;
		default:
			miSexo = "mujer";
			break;
		}
		
		int[] valores = {0,0,0,0,0,0,0,0};
		
		String miResultado = enviarDatos(tipo, miSexo, miNacional, miExtranjero, miEdad, miEspanha, miEstudios, miDistrito, miSeccion, miZona);
		//System.out.println(">"+miResultado+"<");
		if (miResultado.length()>2)
			miResultado = miResultado.substring(0, miResultado.length()-2);
		String [] r = miResultado.split("@");
		
		for (int j=0; j<r.length; j++) {
			String [] s = r[j].split("#");
			switch (s[0]) {
			case "SIN ESTUDIOS":
				valores[0] = Integer.parseInt(s[1]);
				break;
			case "ESTUDIOS PRIMARIOS":
				valores[1] = Integer.parseInt(s[1]);
				break;
			case "ENSEANZA PROFESIONAL 2 GRADO (2 CICLO)":
				valores[2] = Integer.parseInt(s[1]);
				break;
			case "ENSEANZA SECUNDARIA (1 CICLO)":
				valores[3] = Integer.parseInt(s[1]);
				break;
			case "ENSEANZA PROFESIONAL SUPERIOR":
				valores[4] = Integer.parseInt(s[1]);
				break;
			case "ENSEANZA SECUNDARIA (2 CICLO)":
				valores[5] = Integer.parseInt(s[1]);
				break;
			case "UNIVERSITARIOS O SUPERIORES":
				valores[6] = Integer.parseInt(s[1]);
				break;
			case "DESCONOCIDO":
				valores[7] = Integer.parseInt(s[1]);
				break;
			}
		}
		
		dataset.addValue(valores[0], "Poblacin", "Sin estudios");
		dataset.addValue(valores[1], "Poblacin", "Estudios Primarios");
		dataset.addValue(valores[2], "Poblacin", "Enseanza Profesional de 2 Grado (Segundo Ciclo)");
		dataset.addValue(valores[3], "Poblacin", "Enseanza Secundaria (Primer Ciclo)");
		dataset.addValue(valores[4], "Poblacin", "Enseanza Profesional Superior");
		dataset.addValue(valores[5], "Poblacin", "Enseanza Secundaria (Segundo Ciclo)");
		dataset.addValue(valores[6], "Poblacin", "Estudios Universitarios o superiores");
		dataset.addValue(valores[7], "Poblacin", "Desconocido");
		
		// Creo la grfica
		JFreeChart chart = ChartFactory.createBarChart(title, null, "Poblacin", dataset, PlotOrientation.VERTICAL, false, true, false);
		// Configuro la inclinacin del texto del eje inferior
		CategoryPlot plot = (CategoryPlot) chart.getPlot();
		CategoryAxis domainAxis = plot.getDomainAxis();
		domainAxis.setCategoryLabelPositions(CategoryLabelPositions.UP_45);
		// Configuro el color y estilo de las barras
		BarRenderer renderer = (BarRenderer) plot.getRenderer();
		renderer.setSeriesPaint(0, colorEstudios);
		renderer.setBarPainter(new StandardBarPainter());
		// Creo el resultado, aado los valores y lo devuelvo
		ResultadoConsulta miResultadoConsulta = new ResultadoConsulta();
		miResultadoConsulta.setChart(chart);
		int poblacionCons = 0;
		for (int i=0; i<valores.length; i++) {
			poblacionCons += valores[i];
		}
		miResultadoConsulta.setPoblacionConsulta(poblacionCons);
		if (poblacion==-1)
			poblacion = consultaPoblacionTotal();
		miResultadoConsulta.setPoblacion(poblacion);
		return miResultadoConsulta;
	}
	
	/**
	 * Devuelve la poblacin total del padrn
	 * 
	 * @return Integer con la poblacin total.
	 */
	private int consultaPoblacionTotal() {
		int valor = -1;
		String miResultado = enviarDatos("count", null, null, null, null, false, null, -1, -1, null);
		if (miResultado.length()>2)
			miResultado = miResultado.substring(0, miResultado.length()-2);
		valor = Integer.parseInt(miResultado);
		return valor;
	}

	/**
	 * Enva los parmetros de consulta a un script PHP para construir una cadena en base al resultado.
	 * 
	 * @param tipo Tipo de consulta (edad, estudios, sexo, procedencia nacional, procedencia extranjera)
	 * @param sexo Sexo.
	 * @param nacional Provincia.
	 * @param extranjero Pas.
	 * @param edad Edad.
	 * @param espanha True si es para cualquier provincia de Espaa, false en cualquier otro caso.
	 * @param estudios Estudios.
	 * @param distrito Distrito. -1 para "ninguno".
	 * @param seccion Seccin. -1 para "ninguna".
	 * @param zona Zona Estadstica.
	 * @return
	 */
    private String enviarDatos(String tipo, String sexo, String nacional, String extranjero, String edad, boolean espanha, String estudios,
    		int distrito, int seccion, String zona) {
        try {
            //se forma la cadena con los datos a enviar
            String dataString ="tipo="+tipo;
            if (sexo!=null) {
            	dataString += "&sex="+sexo;
            }
            if (nacional!=null) {
            	dataString += "&nac="+nacional;
            }
            if (extranjero!=null) {
            	dataString += "&extr="+extranjero;
            }
            if (edad!=null) {
            	dataString += "&edad="+edad;
            }
            if (espanha) {
            	dataString += "&esp=1";
            }
            if (estudios!=null) {
            	dataString += "&est="+estudios;
            }
            if (distrito!=-1) {
            	dataString += "&dto="+distrito;
            }
            if (seccion!=-1) {
            	dataString += "&sec="+seccion;
            }
            if (zona!=null) {
            	dataString += "&zona="+zona;
            }
            URLEncoder.encode(dataString, "UTF-8");

            // Send data
            URL url = new URL(phpURL);
            URLConnection conn = url.openConnection();
            conn.setDoOutput(true);
            OutputStreamWriter wr = new OutputStreamWriter(conn.getOutputStream());
            wr.write(dataString);
            wr.flush();
            wr.close();
            //se obtiene la respuesta del servidor
            BufferedReader rd = new BufferedReader(new InputStreamReader(conn.getInputStream()));
            String linea, tmp = "";
            while (( linea = rd.readLine()) != null)
            	tmp+=linea;
            rd.close();

            return tmp;
        } catch (IOException ex) {
        	ex.printStackTrace();
            return "";
        }
    }
}